Package: multicrispr
Title: Multi-locus multi-purpose Crispr/Cas design
Version: 1.22.0
Encoding: UTF-8
Authors@R: c(person("Aditya",   "Bhagwat",    NULL, "aditya.bhagwat@uni-marburg.de",   c("aut", "cre")),
            person( "Richie",   "´Cotton",    NULL, "richierocks@gmail.com",            "aut"),
            person( "Rene",     "Wiegandt",   NULL, "rene.wiegandt@mpi-bn.mpg.de",      "ctb"),
            person( "Mette",    "Bentsen",    NULL, "mette.bentsen@mpi-bn.mpg.de",      "ctb"),
            person( "Jens",     "Preussner",  NULL, "jens.preussner@mpi-bn.mpg.de",     "ctb"),
            person( "Michael",  "Lawrence",   NULL, "lawrence.michael@gene.com",        "ctb"),
            person( "Hervé",    "Pagès",      NULL, "hpages.on.github@gmail.com",       "ctb"),
            person( "Johannes", "Graumann",   NULL, 'johannes.graumann@uni-marburg.de', "sad"),
            person( "Mario",    "Looso",      NULL, "mario.looso@mpi-bn.mpg.de",        c("sad","rth")))
Description: This package is for designing Crispr/Cas9 and Prime Editing experiments.
             It contains functions to (1) define and transform genomic targets, (2) find spacers
             (4) count offtarget (mis)matches, and (5) compute Doench2016/2014 targeting efficiency. 
             Care has been taken for multicrispr to scale well towards large target sets, 
             enabling the design of large Crispr/Cas9 libraries.
License: GPL-2
RoxygenNote: 7.3.2
Depends: R (>= 4.0)
Imports: BiocGenerics, Biostrings, BSgenome, CRISPRseek, data.table,
        Seqinfo, GenomicFeatures, GenomicRanges, ggplot2, grid,
        karyoploteR, magrittr, methods, parallel, plyranges, Rbowtie,
        reticulate, rtracklayer, stats, stringi, tidyr, tidyselect,
        utils
Suggests: AnnotationHub, BiocStyle, BSgenome.Hsapiens.UCSC.hg38,
        BSgenome.Mmusculus.UCSC.mm10,
        BSgenome.Scerevisiae.UCSC.sacCer1, ensembldb, IRanges,
        GenomeInfoDb, knitr, magick, rmarkdown, testthat,
        TxDb.Mmusculus.UCSC.mm10.knownGene
VignetteBuilder: knitr
biocViews: CRISPR, Software
BugReports: https://github.com/bhagwataditya/multicrispr/issues
URL: https://github.com/bhagwataditya/multicrispr
git_url: https://git.bioconductor.org/packages/multicrispr
git_branch: RELEASE_3_23
git_last_commit: 5151ca8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:57:16 UTC; biocbuild
Author: Aditya Bhagwat [aut, cre],
  Richie ´Cotton [aut],
  Rene Wiegandt [ctb],
  Mette Bentsen [ctb],
  Jens Preussner [ctb],
  Michael Lawrence [ctb],
  Hervé Pagès [ctb],
  Johannes Graumann [sad],
  Mario Looso [sad, rth]
Maintainer: Aditya Bhagwat <aditya.bhagwat@uni-marburg.de>
Built: R 4.6.0; ; 2026-04-29 04:52:14 UTC; unix
