# Generated by roxygen2: do not edit by hand

export(add_genome_matches)
export(add_inverse_strand)
export(add_seq)
export(add_target_matches)
export(bed_to_granges)
export(char_to_granges)
export(double_flank)
export(down_flank)
export(dt2gr)
export(extend)
export(extend_for_pe)
export(extend_pe_to_gg)
export(extract_matchranges)
export(extract_subranges)
export(find_gg)
export(find_primespacers)
export(find_spacers)
export(genefile_to_granges)
export(genes_to_granges)
export(gr2dt)
export(has_been_indexed)
export(index_genome)
export(index_targets)
export(plot_intervals)
export(plot_karyogram)
export(read_ranges)
export(score_ontargets)
export(up_flank)
export(write_ranges)
importFrom(BSgenome,getBSgenome)
importFrom(BSgenome,getSeq)
importFrom(BiocGenerics,"end<-")
importFrom(BiocGenerics,"start<-")
importFrom(BiocGenerics,"strand<-")
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,invertStrand)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,strand)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,complement)
importFrom(Biostrings,vcountPDict)
importFrom(Biostrings,vmatchPattern)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicRanges,"mcols<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,granges)
importFrom(GenomicRanges,mcols)
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,"seqnames<-")
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqlevelsInUse)
importFrom(Seqinfo,seqnames)
importFrom(Seqinfo,sortSeqlevels)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,setnafill)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grid,arrow)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,add)
importFrom(magrittr,and)
importFrom(magrittr,extract)
importFrom(magrittr,extract2)
importFrom(magrittr,set_names)
importFrom(methods,as)
importFrom(methods,is)
importFrom(reticulate,py_module_available)
importFrom(stats,complete.cases)
importFrom(stringi,stri_detect_fixed)
importFrom(stringi,stri_detect_regex)
importFrom(stringi,stri_locate_all_fixed)
importFrom(stringi,stri_locate_all_regex)
importFrom(stringi,stri_replace_first_fixed)
importFrom(stringi,stri_startswith_fixed)
importFrom(tidyr,separate_rows)
importFrom(tidyselect,starts_with)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
